@@ -4,6 +4,7 @@
  *        Authors: David Bauer, Guruprasad Kora, Nagiza. F. Samatova, 
  *                            Srikanth Yoginath.
  *     Contact: Nagiza F. Samatova; (865) 241-4351; samatovan@ornl.gov
+ *     Contact: Guruprasad Kora; (865) 576-6210; koragh@ornl.gov
  *                 Computer Science and Mathematics Division
  *             Oak Ridge National Laboratory, Oak Ridge TN 37831 
  *                   (C) 2004 All Rights Reserved
@@ -37,8 +38,15 @@
 #include <R_ext/Random.h>
 #include <R_ext/PrtUtil.h>
 
+#include <dlfcn.h>
+
+
 #define DONT_SPAWN_R
 
+#ifdef DONT_SPAWN_R
+#define Rprintf printf
+#endif
+
 #define CHECKFAULT_TAG 1202	/* Tag used to check fault before data send  */
 #define NUMMATRICES_TAG 202	/* Tag used to number of Results to collect  */
 #define RECVRESULT_TAG 300	/* Tag used to receive results */
@@ -47,7 +55,9 @@
 
 #define MPINULL MPI_STATUS_IGNORE
 
-/*#define DEBUG_RSCALAPACK*/
+#if 0
+#define DEBUG_RSCALAPACK
+#endif
 
 #ifndef DEBUG_RSCALAPACK
 #define D_Rprintf(x)
@@ -82,6 +92,7 @@ int PA_UnpackInput(SEXP sxInputVector, int *ipDims, double **dppA,
 SEXP PA_Exec(SEXP scriptLocn, SEXP sxInputVector);
 int PA_SendData(int [], double [], double []);
 SEXP PA_RecvResult(int []);
+SEXP PA_GridInfo();
 
 int PA_GetTwoDims(SEXP , int *);
 int PA_SetDim(SEXP ,int , int *);
